;;;; "Basic Choreography" -- just show off audio/video

(ns com.aurellem.run.basic-choreo
  (:use (com.aurellem.gb saves gb-driver util constants
                         items vbm characters money
                         rlm-assembly))
  (:use (com.aurellem.run util music title save-corruption
                          bootstrap-0 bootstrap-1 image
                          ram-display final-cut))
  (:require clojure.string)
  (:import [com.aurellem.gb.gb_driver SaveState])
  (:import java.io.File))

;; take over game
;; [X] silence noise
;; [X] preload audio/video programs
;; [X] run picture program to display pinkie pie mark
;; [X] play pony song 

(defn-memo silence-noise
  ([] (silence-noise (launch-main-bootstrap-program)))
  ([script]
     (->> script
          (write-RAM 0xFF1A [0 0 0])
          (relocate-main-bootstrap))))

(def music-program-start 0xC000)
(def image-program-start 0xCE00)

(defn-memo write-pony-music
  ([] (write-pony-music (silence-noise)))
  ([script]
     (let [bytes (midi-bytes
                  pony-csv
                  music-program-start
                  0xC400
                  0xC800
                  0xCC00)]
       (->> script
            (write-RAM (:address (:voice-1 bytes))
                       (:data    (:voice-1 bytes)))
            (write-RAM (:address (:voice-2 bytes))
                       (:data    (:voice-2 bytes)))
            (write-RAM (:address (:noise bytes))
                       (:data    (:noise bytes)))
            (write-RAM (:address (:kernel bytes))
                       (:data    (:kernel bytes)))))))

(defn-memo write-image-display-program
  ([] (write-image-display-program (write-pony-music)))
  ([script]
     (let [goto-music
           (flatten
            [0xC3
             (reverse (disect-bytes-2 music-program-start))])]
       (->> script
            (write-RAM
             image-program-start
             (concat
              (display-image-kernel
               image-program-start
               pinkie-pie-mark)
              goto-music))))))
              
(defn-memo activate-media
  ([] (activate-media (write-image-display-program)))
  ([script]
     (->> script
          (transfer-control image-program-start)
          (do-nothing 1800))))

(def basic-render-dir
  (File. user-home "proj/vba-clojure/render/basic"))

(defn basic-steps [] (first (activate-media)))

(defn basic-step-1! []
  (render-files!
   basic-render-dir
   (root) (basic-steps)))

(defn basic-step-2! []
  (generate-composite-frames!
   basic-render-dir))

(defn basic-step-3! [] (final-cut! basic-render-dir))
